#include <SPI.h>
#include <Ethernet.h>
#include <ULCD.h>

const byte CURRENT = 65;                                 // Регистр текущего объекта ULCD
byte Yandex[] = { 213, 180, 204, 137 };                  // info.maps.yandex.net
byte GisMeteo[] = { 92, 241, 171, 120 };                 // informer.gismeteo.ru
int b1 = 0;                                              // Байт для анализа цвета информера пробок
int b2 = 0;                                              // Байт для анализа цвета информера пробок
String Response = "";                                    // Строка полученная от сервера
Client Yclient(Yandex, 80);                              // Клиент для подключения к Yandex
Client Mclient(GisMeteo, 80);                            // Клиент для подключения к GisMeteo
boolean time = false;                                    // Признак запроса время/траффик
ULCD lcd;

void setup() {
  byte mac[] = { 0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED };   // МАС- адрес Ардуино
  byte ip[] = { 192, 168, 0, 110 };                      // IP - адрес Ардуино
  byte gateway[] = { 192, 168, 0, 1 };                   // Шлюз для выхода в интернет
  byte subnet[] = { 255, 255, 255, 0 };                  // Маска подсети
  Ethernet.begin(mac, ip, gateway, subnet);              // Инициализация Ethernet
  Serial.begin(9600);                                    // Инициализация UART для выдачи отладочных сообщений
  ULCDInit();                                            // Создаем виджеты, загружаем шрифты и картинки для начала работы
  GetTime();
}

void loop() {
  
  if (Mclient.available()) {
      Response.concat((char)Mclient.read());
      if (Response.endsWith("latitude")) Response = "";
      if (Response.endsWith("<PHENOMENA")) Response = "";
      if (Response.endsWith("<PRESSURE")) {  
        b1 = Response.indexOf("cloudiness");
        Serial.println("Cloud: " + Response.substring(b1+12, b1+13));
        b2 = Response.indexOf("precipitation", b1+13);
        Serial.println("Rainfall: " + Response.substring(b2+15, b2+16));
        if (Response.substring(b2+15, b2+16) == "4" || Response.substring(b2+15, b2+16) == "5")
          lcd.REPAINT_BMP(12, 16);
         else if (Response.substring(b2+15, b2+16) == "8") 
          lcd.REPAINT_BMP(12, 17);
         else if (Response.substring(b1+12, b1+13) == "0")
          lcd.REPAINT_BMP(12, 13);
         else if (Response.substring(b1+12, b1+13) == "3")
          lcd.REPAINT_BMP(12, 15);
         else if (Response.substring(b1+12, b1+13) == "1" || Response.substring(b1+12, b1+13) == "2")
          lcd.REPAINT_BMP(12, 14);
        Response = "";
      }
      if (Response.endsWith("<WIND")) {  
        Mclient.stop();
        Serial.println("Disconnecting from GisMeteo");   
        b1 = Response.indexOf("<TEMPERATURE");
        b2 = Response.indexOf("/>", b1+1);
        Response = Response.substring(b1+18, b2);
        Response = Response.replace((char)34, "");                //Удаляем кавычки
        Response = Response.replace(" min=", "...");              //Удаляем лишний кусочек текста и ставим многоточие 
        Serial.println("Temperature: " + Response);
        lcd.REPAINT_TEXT(11, 9, Response);
        Response = "";
        Serial.println(freeRam());
        GetTraffic();
      }
  }  
  
 if (Yclient.available() && time) { 
    Response.concat((char)Yclient.read());
    if (Response.endsWith("GMT")) { 
      b1 = Response.indexOf("GMT");
      Serial.println(Response.substring(b1-26,b1-4)); 
      Yclient.stop();
      Serial.println("Disconnecting from Yandex");   
      lcd.REPAINT_TEXT(4, 2, Response.substring(b1-26,b1-4));
      Response = "";
      time = false;
      GetMeteo();
    } 
  }
   
  if (Yclient.available() && !time) {
    byte b3 = Yclient.read();
    if ((b1 == 0x3f)&&(b2 == 0xbb)&&(b3 == 0x00)) {
        Yclient.stop();
        Serial.println("Traffic: Green");
        Serial.println("Disconnecting from Yandex");
        lcd.REPAINT_BMP(8, 5);
      }
    if ((b1 == 0xff)&&(b2 == 0xa4)&&(b3 == 0x00)) {
        Yclient.stop();
        Serial.println("Traffic: Yellow");
        Serial.println("Disconnecting from Yandex");
        lcd.REPAINT_BMP(8, 6);
      }
    if ((b1 == 0xff)&&(b2 == 0x2a)&&(b3 == 0x00)) {
        Yclient.stop();
        Serial.println("Traffic: Red");
        Serial.println("Disconnecting from Yandex");
        lcd.REPAINT_BMP(8, 7);
      }  
    b1 = b2;
    b2 = b3;
  }

  if (!Mclient.connected() && !Yclient.connected()) {
      Yclient.stop();
      Mclient.stop();
      delay(30000);
      Response = "";
      GetTime();
    } 
}

void ULCDInit() {
  SPI.setDataMode(SPI_MODE3);
  lcd.digitalPortWrite(8);                                        //Создаем контейнер
  lcd.SEND_REG(2, CURRENT);                                       //Сохраняем контейнер в регистр R2 для хранения строки времени
  lcd.digitalPortWrite(8);                                        //Создаем контейнер
  lcd.SEND_REG(9, CURRENT);                                       //Сохраняем контейнер в регистр R9 для хранения строки погоды (температура)
  lcd.LOAD_FONT(0, "font20.fnt");                                 //Загружаем шрифт 20 с flash памяти TE-ULCD в регистр R0
  lcd.LOAD_FONT(10, "font12.fnt");                                //Загружаем шрифт 12 с flash памяти TE-ULCD в регистр R10
  lcd.LOAD_PICTURE(1, "back.bmp");                                //Загружаем картинки и сохраняем их в регистры...
  lcd.LOAD_PICTURE(5, "G.bmp");
  lcd.LOAD_PICTURE(6, "Y.bmp");
  lcd.LOAD_PICTURE(7, "R.bmp");
  lcd.LOAD_PICTURE(13, "clear.bmp");
  lcd.LOAD_PICTURE(14, "cloud.bmp");
  lcd.LOAD_PICTURE(15, "mcloud.bmp");
  lcd.LOAD_PICTURE(16, "rain.bmp");
  lcd.LOAD_PICTURE(17, "storm.bmp"); 
  lcd.CR_WID(1);                                                  //Создаем виджет Фрейм
  lcd.SEND_REG(3, CURRENT);                                       //Сохраняем фрейм в регистр R3 
  lcd.CR_WID(5);                                                  //Создаем виджет BitMap (для фона)
  lcd.SET_SIZE(320, 240);                                         //Устанавливаем размер BitMap
  lcd.SET_POS(0, 0);                                              //Устанавливаем позицию
  lcd.SET_PICTURE(1);                                             //Подготавливаем картинку для виджета
  lcd.CR_WID(5);                                                  //Создаем виджет BitMap (для пробок)
  lcd.SET_SIZE(24, 20);                                           //Устанавливаем размер BitMap (для пробок)
  lcd.SET_POS(296, 136);                                          //Устанавливаем позицию для BitMap (для пробок)
  lcd.SEND_REG(8, CURRENT);                                       //Сохраняем в регистр R8 
  lcd.CR_WID(5);                                                  //Создаем виджет BitMap (для погоды)
  lcd.SET_SIZE(64, 64);                                           //Устанавливаем размер BitMap (для погоды)
  lcd.SET_POS(256, 176);                                          //Устанавливаем позицию для BitMap (для погоды)
  lcd.SEND_REG(12, CURRENT);                                      //Сохраняем в регистр R12
  lcd.CR_WID(7);                                                  //Создаем виджет Текст (для времени)
  lcd.SET_FONT_COLOR(134, 177, 230);                              //Устанавливаем цвет текста виджета (для времени)
  lcd.SET_BACK_COLOR(70, 112, 188);                               //Устанавливаем цвет фона виджета (для времени)
  lcd.SET_FONT(0);                                                //Установить шрифт (для времени)
  lcd.SET_POS(10, 90);                                            //Устанавливаем позицию для Текста (для времени)
  lcd.SEND_REG(4, CURRENT);                                       //Сохраняем в регистр R4 
  lcd.CR_WID(7);                                                  //Создаем виджет Текст (для температуры)
  lcd.SET_FONT_COLOR(134, 177, 230);                              //Устанавливаем цвет текста виджета (для температуры)
  lcd.SET_BACK_COLOR(58, 93, 159);                                //Устанавливаем цвет фона виджета (для температуры)
  lcd.SET_FONT(10);                                               //Установить шрифт (для температуры)
  lcd.SET_POS(270, 160);                                          //Устанавливаем позицию для Текста (для температуры)
  lcd.SEND_REG(11, CURRENT);                                      //Сохраняем в регистр R11
  lcd.SET_MAIN(3);
  SPI.setDataMode(SPI_MODE0);
}

void GetMeteo() {                                                  //Процедура подключения к серверу GisMeteo
  Serial.println("Connecting to informer.gismeteo.ru");
  if (Mclient.connect()) {
    Serial.println("Connected to GisMeteo");
    Mclient.println("GET http://informer.gismeteo.ru/xml/88916_1.xml");  //Погода для Ивантеевки
  } 
  else {
    Serial.println("Connection failed");
  }
}

void GetTime() {                                              //Процедура подключения к серверу Yandex
  Serial.println("Connecting to info.maps.yandex.net");
  if (Yclient.connect()) {
    Serial.println("Connected to Yandex");
    time = true; 
    Yclient.println("GET http://info.maps.yandex.net/traffic/moscow HTTP/1.1"); 
    Yclient.println();
  } 
  else {
    Serial.println("Connection failed");
  }
}

void GetTraffic() {                                           //Процедура подключения к серверу Yandex
  b2 = 0;                                                     //Этого достаточно для Яндекс.пробок
  Serial.println("Connecting to info.maps.yandex.net");
  if (Yclient.connect()) {
    Serial.println("Connected to Yandex");
    Yclient.println("GET http://info.maps.yandex.net/traffic/moscow/current_traffic_88.gif HTTP/1.1");
    Yclient.println("Accept: image/gif");
    Yclient.println("Accept-Language: en-US;");
    Yclient.println("Cache-Control: max-age=0");
    Yclient.println("Host: info.maps.yandex.net");
    Yclient.println("User-Agent: Chrome");
    Yclient.println();
  } 
  else {
    Serial.println("Connection failed");
  }
}

int freeRam () {
  extern int __heap_start, *__brkval; 
  int v; 
  return (int) &v - (__brkval == 0 ? (int) &__heap_start : (int) __brkval); 
}
